@@ -47,7 +47,6 @@ def to_action(dp, dic):
     ofp = dp.ofproto
     parser = dp.ofproto_parser
 
-    result = None
     action_type = dic.get('type')
     if action_type == 'OUTPUT':
         out_port = int(dic.get('port', ofp.OFPP_ANY))
@@ -94,7 +93,7 @@ def to_action(dp, dic):
     elif action_type == 'POP_PBB':
         result = parser.OFPActionPopPbb()
     else:
-        LOG.debug('Unknown action type: %s' % action_type)
+        result = None
 
     return result
 
@@ -153,7 +152,7 @@ def action_to_str(act):
     elif action_type == ofproto_v1_3.OFPAT_PUSH_MPLS:
         buf = 'PUSH_MPLS:' + str(act.ethertype)
     elif action_type == ofproto_v1_3.OFPAT_POP_MPLS:
-        buf = 'POP_MPLS'
+        buf = 'POP_MPLS:' + str(act.ethertype)
     elif action_type == ofproto_v1_3.OFPAT_SET_QUEUE:
         buf = 'SET_QUEUE:' + str(act.queue_id)
     elif action_type == ofproto_v1_3.OFPAT_GROUP:
@@ -259,6 +258,7 @@ def to_match(dp, attrs):
                'ipv6_exthdr': int}
 
     match_append = {'in_port': match.set_in_port,
+                    'in_phy_port': match.set_in_phy_port,
                     'dl_src': match.set_dl_src_masked,
                     'dl_dst': match.set_dl_dst_masked,
                     'dl_type': match.set_dl_type,
@@ -335,6 +335,8 @@ def to_match(dp, attrs):
                 ip = value[0]
                 mask = value[1]
                 match_append[key](ip, mask)
+            elif key == 'ipv6_nd_target':
+                match_append[key](value[0])
             elif key == 'tp_src' or key == 'tp_dst' or \
                     key == 'tcp_src' or key == 'tcp_dst' or \
                     key == 'udp_src' or key == 'udp_dst':
@@ -370,7 +372,7 @@ def to_match_tpsrc(value, match, rest):
     match_append = {inet.IPPROTO_TCP: match.set_tcp_src,
                     inet.IPPROTO_UDP: match.set_udp_src}
 
-    nw_proto = rest.get('nw_proto', rest.get('ip_proto', 0))
+    nw_proto = int(rest.get('nw_proto', rest.get('ip_proto', 0)))
     if nw_proto in match_append:
         match_append[nw_proto](value)
 
@@ -381,7 +383,7 @@ def to_match_tpdst(value, match, rest):
     match_append = {inet.IPPROTO_TCP: match.set_tcp_dst,
                     inet.IPPROTO_UDP: match.set_udp_dst}
 
-    nw_proto = rest.get('nw_proto', rest.get('ip_proto', 0))
+    nw_proto = int(rest.get('nw_proto', rest.get('ip_proto', 0)))
     if nw_proto in match_append:
         match_append[nw_proto](value)
 
@@ -496,6 +498,8 @@ def match_to_str(ofmatch):
             value = match_ip_to_str(match_field.value, match_field.mask)
         elif key == 'ipv6_src' or key == 'ipv6_dst':
             value = match_ipv6_to_str(match_field.value, match_field.mask)
+        elif key == 'ipv6_nd_target':
+            value = match_ipv6_to_str(match_field.value, None)
         elif key == 'metadata':
             value = ('%d/%d' % (match_field.value, match_field.mask)
                      if match_field.mask else '%d' % match_field.value)
@@ -889,6 +893,33 @@ def get_group_desc(dp, waiters):
     return descs
 
 
+def get_port_desc(dp, waiters):
+
+    stats = dp.ofproto_parser.OFPPortDescStatsRequest(dp, 0)
+    msgs = []
+    send_stats_request(dp, stats, waiters, msgs)
+
+    descs = []
+
+    for msg in msgs:
+        stats = msg.body
+        for stat in stats:
+            d = {'port_no': stat.port_no,
+                 'hw_addr': stat.hw_addr,
+                 'name': stat.name,
+                 'config': stat.config,
+                 'state': stat.state,
+                 'curr': stat.curr,
+                 'advertised': stat.advertised,
+                 'supported': stat.supported,
+                 'peer': stat.peer,
+                 'curr_speed': stat.curr_speed,
+                 'max_speed': stat.max_speed}
+            descs.append(d)
+    descs = {str(dp.id): descs}
+    return descs
+
+
 def mod_flow_entry(dp, flow, cmd):
     cookie = int(flow.get('cookie', 0))
     cookie_mask = int(flow.get('cookie_mask', 0))
@@ -988,6 +1019,19 @@ def mod_group_entry(dp, group, cmd):
     dp.send_msg(group_mod)
 
 
+def mod_port_behavior(dp, port_config):
+    port_no = int(port_config.get('port_no', 0))
+    hw_addr = port_config.get('hw_addr')
+    config = int(port_config.get('config', 0))
+    mask = int(port_config.get('mask', 0))
+    advertise = int(port_config.get('advertise'))
+
+    port_mod = dp.ofproto_parser.OFPPortMod(
+        dp, port_no, hw_addr, config, mask, advertise)
+
+    dp.send_msg(port_mod)
+
+
 def send_experimenter(dp, exp):
     experimenter = exp.get('experimenter', 0)
     exp_type = exp.get('exp_type', 0)
