@@ -581,6 +581,10 @@ _rf_map = {
 }
 
 
+def get_rf(afi, safi):
+    return _rf_map[(afi, safi)]
+
+
 def pad(bin, len_):
     assert len(bin) <= len_
     return bin + (len_ - len(bin)) * '\0'
@@ -1767,6 +1771,7 @@ class BGPPathAttributeMpReachNLRI(_PathAttribute):
     _VALUE_PACK_STR = '!HBB'  # afi, safi, next hop len
     _ATTR_FLAGS = BGP_ATTR_FLAG_OPTIONAL
     _class_prefixes = ['_BinAddrPrefix']
+    _rd_length = 8
 
     def __init__(self, afi, safi, next_hop, nlri,
                  next_hop_len=0, reserved='\0',
@@ -1802,8 +1807,16 @@ class BGPPathAttributeMpReachNLRI(_PathAttribute):
         while binnlri:
             n, binnlri = addr_cls.parser(binnlri)
             nlri.append(n)
-        if RouteFamily(afi, safi) in (RF_IPv6_UC, RF_IPv6_VPN):
+
+        rf = RouteFamily(afi, safi)
+        if rf == RF_IPv6_UC:
             next_hop = addrconv.ipv6.bin_to_text(next_hop_bin)
+        elif rf == RF_IPv6_VPN:
+            next_hop = addrconv.ipv6.bin_to_text(next_hop_bin[cls._rd_length:])
+            next_hop_len -= cls._rd_length
+        elif rf == RF_IPv4_VPN:
+            next_hop = addrconv.ipv4.bin_to_text(next_hop_bin[cls._rd_length:])
+            next_hop_len -= cls._rd_length
         else:
             next_hop = addrconv.ipv4.bin_to_text(next_hop_bin)
         return {
@@ -1818,12 +1831,22 @@ class BGPPathAttributeMpReachNLRI(_PathAttribute):
     def serialize_value(self):
         # fixup
         self.next_hop_len = len(self._next_hop_bin)
+
+        if RouteFamily(self.afi, self.safi) in (RF_IPv4_VPN, RF_IPv6_VPN):
+            empty_label_stack = '\x00' * self._rd_length
+            next_hop_len = len(self._next_hop_bin) + len(empty_label_stack)
+            next_hop_bin = empty_label_stack
+            next_hop_bin += self._next_hop_bin
+        else:
+            next_hop_len = self.next_hop_len
+            next_hop_bin = self._next_hop_bin
+
         self.reserved = '\0'
 
         buf = bytearray()
         msg_pack_into(self._VALUE_PACK_STR, buf, 0, self.afi,
-                      self.safi, self.next_hop_len)
-        buf += self._next_hop_bin
+                      self.safi, next_hop_len)
+        buf += next_hop_bin
         buf += self.reserved
         binnlri = bytearray()
         for n in self.nlri:
@@ -1910,9 +1933,9 @@ class BGPMessage(packet_base.PacketBase, _TypeDisp):
 
     def __init__(self, type_, len_=None, marker=None):
         if marker is None:
-            self.marker = _MARKER
+            self._marker = _MARKER
         else:
-            self.marker = marker
+            self._marker = marker
         self.len = len_
         self.type = type_
 
@@ -1935,11 +1958,11 @@ class BGPMessage(packet_base.PacketBase, _TypeDisp):
 
     def serialize(self):
         # fixup
-        self.marker = _MARKER
+        self._marker = _MARKER
         tail = self.serialize_tail()
         self.len = self._HDR_LEN + len(tail)
 
-        hdr = bytearray(struct.pack(self._HDR_PACK_STR, self.marker,
+        hdr = bytearray(struct.pack(self._HDR_PACK_STR, self._marker,
                                     self.len, self.type))
         return hdr + tail
 
