@@ -29,6 +29,8 @@ from ryu.services.protocols.bgp.base import SUPPORTED_GLOBAL_RF
 from ryu.services.protocols.bgp import constants as const
 from ryu.services.protocols.bgp.model import OutgoingRoute
 from ryu.services.protocols.bgp.model import SentRoute
+from ryu.services.protocols.bgp.info_base.base import PrefixFilter
+from ryu.services.protocols.bgp.model import ReceivedRoute
 from ryu.services.protocols.bgp.net_ctrl import NET_CONTROLLER
 from ryu.services.protocols.bgp.rtconf.neighbors import NeighborConfListener
 from ryu.services.protocols.bgp.signals.emit import BgpSignalBus
@@ -46,11 +48,17 @@ from ryu.lib.packet.bgp import RF_IPv6_UC
 from ryu.lib.packet.bgp import RF_IPv4_VPN
 from ryu.lib.packet.bgp import RF_IPv6_VPN
 from ryu.lib.packet.bgp import RF_RTC_UC
+from ryu.lib.packet.bgp import get_rf
 
 from ryu.lib.packet.bgp import BGPOpen
 from ryu.lib.packet.bgp import BGPUpdate
 from ryu.lib.packet.bgp import BGPRouteRefresh
 
+from ryu.lib.packet.bgp import BGP_ERROR_CEASE
+from ryu.lib.packet.bgp import BGP_ERROR_SUB_ADMINISTRATIVE_SHUTDOWN
+from ryu.lib.packet.bgp import BGP_ERROR_SUB_CONNECTION_COLLISION_RESOLUTION
+
+
 from ryu.lib.packet.bgp import BGP_MSG_UPDATE
 from ryu.lib.packet.bgp import BGP_MSG_KEEPALIVE
 from ryu.lib.packet.bgp import BGP_MSG_ROUTE_REFRESH
@@ -71,6 +79,9 @@ from ryu.lib.packet.bgp import BGP_ATTR_TYPE_MULTI_EXIT_DISC
 from ryu.lib.packet.bgp import BGP_ATTR_TYPE_COMMUNITIES
 from ryu.lib.packet.bgp import BGP_ATTR_TYPE_EXTENDED_COMMUNITIES
 
+from ryu.lib.packet.bgp import BGPTwoOctetAsSpecificExtendedCommunity
+from ryu.lib.packet.bgp import BGPIPv4AddressSpecificExtendedCommunity
+
 LOG = logging.getLogger('bgpspeaker.peer')
 
 
@@ -187,12 +198,14 @@ class PeerState(object):
         if new_state == const.BGP_FSM_ESTABLISHED:
             self.incr(PeerCounterNames.FSM_ESTB_TRANSITIONS)
             self._established_time = time.time()
+            self._signal_bus.adj_up(self.peer)
             NET_CONTROLLER.send_rpc_notification(
                 'neighbor.up', {'ip_address': self.peer.ip_address}
             )
         # transition from Established to another state
         elif old_state == const.BGP_FSM_ESTABLISHED:
             self._established_time = 0
+            self._signal_bus.adj_down(self.peer)
             NET_CONTROLLER.send_rpc_notification(
                 'neighbor.down', {'ip_address': self.peer.ip_address}
             )
@@ -323,6 +336,18 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
         self._sent_init_non_rtc_update = False
         self._init_rtc_nlri_path = []
 
+        # in-bound filters
+        self._in_filters = self._neigh_conf.in_filter
+
+        # out-bound filters
+        self._out_filters = self._neigh_conf.out_filter
+
+        # Adj-rib-in
+        self._adj_rib_in = {}
+
+        # Adj-rib-out
+        self._adj_rib_out = {}
+
     @property
     def remote_as(self):
         return self._neigh_conf.remote_as
@@ -355,6 +380,26 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
     def med(self):
         return self._neigh_conf.multi_exit_disc
 
+    @property
+    def in_filters(self):
+        return self._in_filters
+
+    @in_filters.setter
+    def in_filters(self, filters):
+        self._in_filters = [f.clone() for f in filters]
+        LOG.debug('set in-filter : %s' % filters)
+        self.on_update_in_filter()
+
+    @property
+    def out_filters(self):
+        return self._out_filters
+
+    @out_filters.setter
+    def out_filters(self, filters):
+        self._out_filters = [f.clone() for f in filters]
+        LOG.debug('set out-filter : %s' % filters)
+        self.on_update_out_filter()
+
     def is_mpbgp_cap_valid(self, route_family):
         if not self.in_established:
             raise ValueError('Invalid request: Peer not in established state')
@@ -405,17 +450,18 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
         enabled = conf_evt.value
         # If we do not have any protocol bound and configuration asks us to
         # enable this peer, we try to establish connection again.
-        LOG.debug('Peer %s configuration update event, enabled: %s.' %
-                  (self, enabled))
         if enabled:
-            if self._protocol:
+            LOG.info('%s enabled' % self)
+            if self._protocol and self._protocol.started:
                 LOG.error('Tried to enable neighbor that is already enabled')
             else:
+                self.state.bgp_state = const.BGP_FSM_CONNECT
                 # Restart connect loop if not already running.
                 if not self._connect_retry_event.is_set():
                     self._connect_retry_event.set()
                     LOG.debug('Starting connect loop as neighbor is enabled.')
         else:
+            LOG.info('%s disabled' % self)
             if self._protocol:
                 # Stopping protocol will eventually trigger connection_lost
                 # handler which will do some clean-up.
@@ -427,10 +473,11 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
                     BGP_ERROR_SUB_ADMINISTRATIVE_SHUTDOWN
                 )
                 self._protocol.stop()
+                self._protocol = None
+                self.state.bgp_state = const.BGP_FSM_IDLE
             # If this peer is not enabled any-more we stop trying to make any
             # connection.
-            LOG.debug('Disabling connect-retry as neighbor was disabled (%s)' %
-                      (not enabled))
+            LOG.debug('Disabling connect-retry as neighbor was disabled')
             self._connect_retry_event.clear()
 
     def on_update_med(self, conf_evt):
@@ -440,6 +487,78 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
             for af in negotiated_afs:
                 self._fire_route_refresh(af)
 
+    def _apply_filter(self, filters, path):
+        block = False
+        blocked_cause = None
+
+        for filter_ in filters:
+            if filter_.ROUTE_FAMILY != path.ROUTE_FAMILY:
+                continue
+
+            policy, is_matched = filter_.evaluate(path)
+            if policy == PrefixFilter.POLICY_PERMIT and is_matched:
+                block = False
+                break
+            elif policy == PrefixFilter.POLICY_DENY and is_matched:
+                block = True
+                blocked_cause = filter_.prefix + ' - DENY'
+                break
+
+        return block, blocked_cause
+
+    def _apply_in_filter(self, path):
+        return self._apply_filter(self._in_filters, path)
+
+    def _apply_out_filter(self, path):
+        return self._apply_filter(self._out_filters, path)
+
+    def on_update_in_filter(self):
+        LOG.debug('on_update_in_filter fired')
+        for received_path in self._adj_rib_in.itervalues():
+            LOG.debug('received_path: %s' % received_path)
+            path = received_path.path
+            nlri_str = path.nlri.formatted_nlri_str
+            block, blocked_reason = self._apply_in_filter(path)
+            if block == received_path.filtered:
+                LOG.debug('block situation not changed: %s' % block)
+                continue
+            elif block:
+                # path wasn't blocked, but must be blocked by this update
+                path = sent_route.path.clone(for_withdrawal=True)
+                LOG.debug('withdraw %s because of in filter update'
+                          % nlri_str)
+            else:
+                # path was blocked, but mustn't be blocked by this update
+                LOG.debug('learn blocked %s because of in filter update'
+                          % nlri_str)
+            received_path.filtered = block
+            tm = self._core_service.table_manager
+            tm.learn_path(path)
+
+    def on_update_out_filter(self):
+        LOG.debug('on_update_out_filter fired')
+        for sent_path in self._adj_rib_out.itervalues():
+            LOG.debug('sent_path: %s' % sent_path)
+            path = sent_path.path
+            nlri_str = path.nlri.formatted_nlri_str
+            block, blocked_reason = self._apply_out_filter(path)
+            if block == sent_path.filtered:
+                LOG.debug('block situation not changed: %s' % block)
+                continue
+            elif block:
+                # path wasn't blocked, but must be blocked by this update
+                withdraw_clone = sent_route.path.clone(for_withdrawal=True)
+                outgoing_route = OutgoingRoute(withdraw_clone)
+                LOG.debug('send withdraw %s because of out filter update'
+                          % nlri_str)
+            else:
+                # path was blocked, but mustn't be blocked by this update
+                outgoing_route = OutgoingRoute(path)
+                LOG.debug('send blocked %s because of out filter update'
+                          % nlri_str)
+            sent_path.filtered = block
+            self.enque_outgoing_msg(outgoing_route)
+
     def __str__(self):
         return 'Peer(ip: %s, asn: %s)' % (self._neigh_conf.ip_address,
                                           self._neigh_conf.remote_as)
@@ -483,19 +602,31 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
         Also, checks if any policies prevent sending this message.
         Populates Adj-RIB-out with corresponding `SentRoute`.
         """
+
+        path = outgoing_route.path
+        block, blocked_cause = self._apply_out_filter(path)
+
+        nlri_str = outgoing_route.path.nlri.formatted_nlri_str
+        sent_route = SentRoute(outgoing_route.path, self, block)
+        self._adj_rib_out[nlri_str] = sent_route
+        self._signal_bus.adj_rib_out_changed(self, sent_route)
+
         # TODO(PH): optimized by sending several prefixes per update.
         # Construct and send update message.
-        update_msg = self._construct_update(outgoing_route)
-        self._protocol.send(update_msg)
-        # Collect update statistics.
-        self.state.incr(PeerCounterNames.SENT_UPDATES)
+        if not block:
+            update_msg = self._construct_update(outgoing_route)
+            self._protocol.send(update_msg)
+            # Collect update statistics.
+            self.state.incr(PeerCounterNames.SENT_UPDATES)
+        else:
+            LOG.debug('prefix : %s is not sent by filter : %s'
+                      % (path.nlri, blocked_cause))
 
         # We have to create sent_route for every OutgoingRoute which is
         # not a withdraw or was for route-refresh msg.
         if (not outgoing_route.path.is_withdraw and
                 not outgoing_route.for_route_refresh):
             # Update the destination with new sent route.
-            sent_route = SentRoute(outgoing_route.path, self)
             tm = self._core_service.table_manager
             tm.remember_sent_route(sent_route)
 
@@ -581,7 +712,7 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
         update = BGPUpdate(path_attributes=[mpunreach_attr])
         self.enque_outgoing_msg(update)
 
-    def _session_next_hop(self, route_family):
+    def _session_next_hop(self, path):
         """Returns nexthop address relevant to current session
 
         Nexthop used can depend on capabilities of the session. If VPNv6
@@ -589,6 +720,13 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
         IPv4 mapped IPv6 address. In other cases we can use connection end
         point/local ip address.
         """
+        route_family = path.route_family
+
+        if route_family == RF_IPv4_UC and path.nexthop != '0.0.0.0':
+            return path.nexthop
+        if route_family == RF_IPv6_UC and path.nexthop != '::':
+            return path.nexthop
+
         # By default we use BGPS's interface IP with this peer as next_hop.
         if self._neigh_conf.next_hop:
             next_hop = self._neigh_conf.next_hop
@@ -638,7 +776,7 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
             # By default we use BGPS's interface IP with this peer as next_hop.
             # TODO(PH): change to use protocol's local address.
             # next_hop = self.host_bind_ip
-            next_hop = self._session_next_hop(path.route_family)
+            next_hop = self._session_next_hop(path)
             # If this is a iBGP peer.
             if not self.is_ebgp_peer() and path.source is not None:
                 # If the path came from a bgp peer and not from NC, according
@@ -850,8 +988,7 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
         # Stop connect_loop retry timer as we are now connected
         if self._protocol and self._connect_retry_event.is_set():
             self._connect_retry_event.clear()
-            LOG.debug('Connect retry event for %s is now set: %s' %
-                      (self, self._connect_retry_event.is_set()))
+            LOG.debug('Connect retry event for %s is cleared' % self)
 
         if self._protocol and self.outgoing_msg_event.is_set():
             # Start processing sink.
@@ -892,7 +1029,8 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
             # If existing protocol is already established, we raise exception.
             if self.state.bgp_state != const.BGP_FSM_IDLE:
                 LOG.debug('Currently in %s state, hence will send collision'
-                          ' Notification to close this protocol.')
+                          ' Notification to close this protocol.'
+                          % self.state.bgp_state)
                 self._send_collision_err_and_stop(proto)
                 return
 
@@ -1153,9 +1291,21 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
                 nexthop=next_hop
             )
             LOG.debug('Extracted paths from Update msg.: %s' % new_path)
-            # Update appropriate table with new paths.
-            tm = self._core_service.table_manager
-            tm.learn_path(new_path)
+
+            block, blocked_cause = self._apply_in_filter(new_path)
+
+            nlri_str = new_path.nlri.formatted_nlri_str
+            received_route = ReceivedRoute(new_path, self, block)
+            self._adj_rib_in[nlri_str] = received_route
+            self._signal_bus.adj_rib_in_changed(self, received_route)
+
+            if not block:
+                # Update appropriate table with new paths.
+                tm = self._core_service.table_manager
+                tm.learn_path(new_path)
+            else:
+                LOG.debug('prefix : %s is blocked by in-bound filter : %s'
+                          % (msg_nlri, blocked_cause))
 
         # If update message had any qualifying new paths, do some book-keeping.
         if msg_nlri_list:
@@ -1202,9 +1352,21 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
                 w_nlri,
                 is_withdraw=True
             )
-            # Update appropriate table with withdraws.
-            tm = self._core_service.table_manager
-            tm.learn_path(w_path)
+
+            block, blocked_cause = self._apply_in_filter(w_path)
+
+            received_route = ReceivedRoute(w_path, self, block)
+            nlri_str = w_nlri.formatted_nlri_str
+            self._adj_rib_in[nlri_str] = received_route
+            self._signal_bus.adj_rib_in_changed(self, received_route)
+
+            if not block:
+                # Update appropriate table with withdraws.
+                tm = self._core_service.table_manager
+                tm.learn_path(w_path)
+            else:
+                LOG.debug('prefix : %s is blocked by in-bound filter : %s'
+                          % (msg_nlri, blocked_cause))
 
     def _extract_and_handle_mpbgp_new_paths(self, update_msg):
         """Extracts new paths advertised in the given update message's
@@ -1282,13 +1444,25 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
                 nexthop=next_hop
             )
             LOG.debug('Extracted paths from Update msg.: %s' % new_path)
-            if msg_rf == RF_RTC_UC \
-                    and self._init_rtc_nlri_path is not None:
-                self._init_rtc_nlri_path.append(new_path)
+
+            block, blocked_cause = self._apply_in_filter(new_path)
+
+            received_route = ReceivedRoute(new_path, self, block)
+            nlri_str = msg_nlri.formatted_nlri_str
+            self._adj_rib_in[nlri_str] = received_route
+            self._signal_bus.adj_rib_in_changed(self, received_route)
+
+            if not block:
+                if msg_rf == RF_RTC_UC \
+                        and self._init_rtc_nlri_path is not None:
+                    self._init_rtc_nlri_path.append(new_path)
+                else:
+                    # Update appropriate table with new paths.
+                    tm = self._core_service.table_manager
+                    tm.learn_path(new_path)
             else:
-                # Update appropriate table with new paths.
-                tm = self._core_service.table_manager
-                tm.learn_path(new_path)
+                LOG.debug('prefix : %s is blocked by in-bound filter : %s'
+                          % (msg_nlri, blocked_cause))
 
         # If update message had any qualifying new paths, do some book-keeping.
         if msg_nlri_list:
@@ -1324,7 +1498,7 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
             )
             return
 
-        w_nlris = mp_unreach_attr.nlri_list
+        w_nlris = mp_unreach_attr.withdrawn_routes
         if not w_nlris:
             # If this is EOR of some kind, handle it
             self._handle_eor(msg_rf)
@@ -1335,9 +1509,20 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
                 w_nlri,
                 is_withdraw=True
             )
-            # Update appropriate table with withdraws.
-            tm = self._core_service.table_manager
-            tm.learn_path(w_path)
+            block, blocked_cause = self._apply_in_filter(w_path)
+
+            received_route = ReceivedRoute(w_path, self, block)
+            nlri_str = w_nlri.formatted_nlri_str
+            self._adj_rib_in[nlri_str] = received_route
+            self._signal_bus.adj_rib_in_changed(self, received_route)
+
+            if not block:
+                # Update appropriate table with withdraws.
+                tm = self._core_service.table_manager
+                tm.learn_path(w_path)
+            else:
+                LOG.debug('prefix : %s is blocked by in-bound filter : %s'
+                          % (w_nlri, blocked_cause))
 
     def _handle_eor(self, route_family):
         """Currently we only handle EOR for RTC address-family.
@@ -1448,7 +1633,7 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
                       demarcation)
 
     def _handle_route_refresh_req(self, afi, safi):
-        rr_af = RouteFamily(afi, safi)
+        rr_af = get_rf(afi, safi)
         self.state.incr(PeerCounterNames.RECV_REFRESH)
 
         # Check if peer has asked for route-refresh for af that was advertised
@@ -1677,7 +1862,7 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
     def connection_made(self):
         """Protocols connection established handler
         """
-        LOG.critical(
+        LOG.info(
             'Connection to peer: %s established',
             self._neigh_conf.ip_address,
             extra={
@@ -1689,7 +1874,7 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
     def connection_lost(self, reason):
         """Protocols connection lost handler.
         """
-        LOG.critical(
+        LOG.info(
             'Connection to peer %s lost, reason: %s Resetting '
             'retry connect loop: %s' %
             (self._neigh_conf.ip_address, reason,
@@ -1701,6 +1886,7 @@ class Peer(Source, Sink, NeighborConfListener, Activity):
         )
         self.state.bgp_state = const.BGP_FSM_IDLE
         if self._protocol:
+            self._protocol.stop()
             self._protocol = None
             # Create new collection for initial RT NLRIs
             self._init_rtc_nlri_path = []
