@@ -61,6 +61,8 @@ from ryu.services.protocols.bgp.rtconf.base import validate_med
 from ryu.services.protocols.bgp.rtconf.base import validate_soo_list
 from ryu.services.protocols.bgp.utils.validation import is_valid_ipv4
 from ryu.services.protocols.bgp.utils.validation import is_valid_old_asn
+from ryu.services.protocols.bgp.info_base.base import Filter
+from ryu.services.protocols.bgp.info_base.base import PrefixFilter
 
 LOG = logging.getLogger('bgpspeaker.rtconf.neighbor')
 
@@ -71,8 +73,10 @@ ENABLED = 'enabled'
 CHANGES = 'changes'
 LOCAL_ADDRESS = 'local_address'
 LOCAL_PORT = 'local_port'
-PEER_NEXT_HOP = 'next_hop'
+PEER_NEXT_HOP = 'peer_next_hop'
 PASSWORD = 'password'
+IN_FILTER = 'in_filter'
+OUT_FILTER = 'out_filter'
 
 # Default value constants.
 DEFAULT_CAP_GR_NULL = True
@@ -85,6 +89,8 @@ DEFAULT_CAP_MBGP_VPNV6 = False
 DEFAULT_HOLD_TIME = 40
 DEFAULT_ENABLED = True
 DEFAULT_CAP_RTC = False
+DEFAULT_IN_FILTER = []
+DEFAULT_OUT_FILTER = []
 
 # Default value for *MAX_PREFIXES* setting is set to 0.
 DEFAULT_MAX_PREFIXES = 0
@@ -164,6 +170,54 @@ def validate_remote_as(asn):
     return asn
 
 
+def valid_prefix_filter(filter_):
+    policy = filter_.get('policy', None)
+    if policy == 'permit':
+        policy = PrefixFilter.POLICY_PERMIT
+    else:
+        policy = PrefixFilter.POLICY_DENY
+    prefix = filter_['prefix']
+    ge = filter_.get('ge', None)
+    le = filter_.get('le', None)
+    return PrefixFilter(prefix, policy, ge=ge, le=le)
+
+PREFIX_FILTER = 'prefix_filter'
+
+SUPPORTED_FILTER_VALIDATORS = {
+    PREFIX_FILTER: valid_prefix_filter
+}
+
+
+def valid_filter(filter_):
+    if isinstance(filter_, Filter):
+        return filter_
+
+    if not isinstance(filter_, dict):
+        raise ConfigTypeError(desc='Invalid filter: %s' % filter_)
+
+    if 'type' not in filter_:
+        raise ConfigTypeError(desc='Invalid filter: %s, needs \'type\' field'
+                              % filter_)
+
+    if not filter_['type'] in SUPPORTED_FILTER_VALIDATORS:
+        raise ConfigTypeError(desc='Invalid filter type: %s, supported filter'
+                              ' types are %s'
+                              % (filter_['type'],
+                                 SUPPORTED_FILTER_VALIDATORS.keys()))
+
+    return SUPPORTED_FILTER_VALIDATORS[filter_['type']](filter_)
+
+
+@validate(name=IN_FILTER)
+def validate_in_filters(filters):
+    return [valid_filter(filter_) for filter_ in filters]
+
+
+@validate(name=OUT_FILTER)
+def validate_out_filters(filters):
+    return [valid_filter(filter_) for filter_ in filters]
+
+
 class NeighborConf(ConfWithId, ConfWithStats):
     """Class that encapsulates one neighbors' configuration."""
 
@@ -180,7 +234,8 @@ class NeighborConf(ConfWithId, ConfWithStats):
                                    ENABLED, MULTI_EXIT_DISC, MAX_PREFIXES,
                                    ADVERTISE_PEER_AS, SITE_OF_ORIGINS,
                                    LOCAL_ADDRESS, LOCAL_PORT,
-                                   PEER_NEXT_HOP, PASSWORD])
+                                   PEER_NEXT_HOP, PASSWORD,
+                                   IN_FILTER, OUT_FILTER])
 
     def __init__(self, **kwargs):
         super(NeighborConf, self).__init__(**kwargs)
@@ -206,6 +261,10 @@ class NeighborConf(ConfWithId, ConfWithStats):
             MAX_PREFIXES, DEFAULT_MAX_PREFIXES, **kwargs)
         self._settings[ADVERTISE_PEER_AS] = compute_optional_conf(
             ADVERTISE_PEER_AS, DEFAULT_ADVERTISE_PEER_AS, **kwargs)
+        self._settings[IN_FILTER] = compute_optional_conf(
+            IN_FILTER, DEFAULT_IN_FILTER, **kwargs)
+        self._settings[OUT_FILTER] = compute_optional_conf(
+            OUT_FILTER, DEFAULT_OUT_FILTER, **kwargs)
 
         # We do not have valid default MED value.
         # If no MED attribute is provided then we do not have to use MED.
@@ -372,6 +431,14 @@ class NeighborConf(ConfWithId, ConfWithStats):
     def rtc_as(self):
         return self._settings[RTC_AS]
 
+    @property
+    def in_filter(self):
+        return self._settings[IN_FILTER]
+
+    @property
+    def out_filter(self):
+        return self._settings[OUT_FILTER]
+
     def exceeds_max_prefix_allowed(self, prefix_count):
         allowed_max = self._settings[MAX_PREFIXES]
         does_exceed = False
