@@ -42,6 +42,7 @@ from ryu.lib.packet.bgp import BGP_CAP_MULTIPROTOCOL
 from ryu.lib.packet.bgp import BGP_CAP_ROUTE_REFRESH
 from ryu.lib.packet.bgp import BGP_ERROR_HOLD_TIMER_EXPIRED
 from ryu.lib.packet.bgp import BGP_ERROR_SUB_HOLD_TIMER_EXPIRED
+from ryu.lib.packet.bgp import get_rf
 
 from ryu.services.protocols.bgp.base import Activity
 from ryu.services.protocols.bgp.base import add_bgp_error_metadata
@@ -173,12 +174,17 @@ class BgpProtocol(Protocol, Activity):
             raise ValueError('Did not yet receive peers open message.')
 
         err_cap_enabled = False
-        local_cap = self.sent_open_msg.caps
-        peer_cap = self.recv_open_msg.caps
+        local_caps = self.sent_open_msg.opt_param
+        peer_caps = self.recv_open_msg.opt_param
+
+        local_cap = [cap for cap in local_caps
+                     if cap.cap_code == BGP_CAP_ENHANCED_ROUTE_REFRESH]
+        peer_cap = [cap for cap in peer_caps
+                    if cap.cap_code == BGP_CAP_ENHANCED_ROUTE_REFRESH]
+
         # Both local and peer should advertise ERR capability for it to be
         # enabled.
-        if (local_cap.get(BGP_CAP_ENHANCED_ROUTE_REFRESH) and
-                peer_cap.get(BGP_CAP_ENHANCED_ROUTE_REFRESH)):
+        if local_cap and peer_cap:
             err_cap_enabled = True
 
         return err_cap_enabled
@@ -217,19 +223,22 @@ class BgpProtocol(Protocol, Activity):
 
     @property
     def negotiated_afs(self):
-        local_caps = self.sent_open_msg.caps
-        remote_caps = self.recv_open_msg.caps
+        local_caps = self.sent_open_msg.opt_param
+        remote_caps = self.recv_open_msg.opt_param
+
+        local_mbgp_cap = [cap for cap in local_caps
+                          if cap.cap_code == BGP_CAP_MULTIPROTOCOL]
+        remote_mbgp_cap = [cap for cap in remote_caps
+                           if cap.cap_code == BGP_CAP_MULTIPROTOCOL]
 
-        local_mbgp_cap = local_caps.get(BGP_CAP_MULTIPROTOCOL)
-        remote_mbgp_cap = remote_caps.get(BGP_CAP_MULTIPROTOCOL)
         # Check MP_BGP capabilities were advertised.
         if local_mbgp_cap and remote_mbgp_cap:
             local_families = {
-                (peer_cap.route_family.afi, peer_cap.route_family.safi)
+                (peer_cap.afi, peer_cap.safi)
                 for peer_cap in local_mbgp_cap
             }
             remote_families = {
-                (peer_cap.route_family.afi, peer_cap.route_family.safi)
+                (peer_cap.afi, peer_cap.safi)
                 for peer_cap in remote_mbgp_cap
             }
             afi_safi = local_families.intersection(remote_families)
@@ -238,7 +247,7 @@ class BgpProtocol(Protocol, Activity):
 
         afs = []
         for afi, safi in afi_safi:
-            afs.append(RouteFamily(afi, safi))
+            afs.append(get_rf(afi, safi))
         return afs
 
     def is_mbgp_cap_valid(self, route_family):
@@ -362,8 +371,9 @@ class BgpProtocol(Protocol, Activity):
         reason = notification.reason
         self._send_with_lock(notification)
         self._signal_bus.bgp_error(self._peer, code, subcode, reason)
-        LOG.error('Sent notification to %r >> %s' % (self._localname(),
-                                                     notification))
+        if len(self._localname):
+            LOG.error('Sent notification to %r >> %s' % (self._localname,
+                                                         notification))
         self._socket.close()
 
     def _send_with_lock(self, msg):
@@ -573,7 +583,6 @@ class BgpProtocol(Protocol, Activity):
         self.sent_open_msg = open_msg
         self.send(open_msg)
         self._peer.connection_made()
-        LOG.debug('Sent open message %s' % open_msg)
 
     def connection_lost(self, reason):
         """Stops all timers and notifies peer that connection is lost.
